import datetime
import uuid
from typing import Union, NamedTuple, Optional, List

from stix2 import Relationship, base, Identity
from stix2.base import _DomainObject, _Observable
from stix2.canonicalization.Canonicalize import canonicalize


class STIXMapperSettings(NamedTuple):
    titan_client: Union['titan_client', None] = None
    api_client: Union['ApiClient', None] = None
    # Resolve GIRs numbers into full names
    girs_names: bool = True
    # Get full reports contents. Applicable to FINTEL and INFOREP
    # as other reports have full content in respective search APIs.
    report_full_content: bool = True
    ioc_opencti_score: Optional[int] = None


def generate_id(
    stix_class: Union[_DomainObject, Relationship, _Observable],
    **id_contributing_properties: str,
) -> str:
    if id_contributing_properties:
        name = canonicalize(id_contributing_properties, utf8=False)
        return f"{stix_class._type}--{uuid.uuid5(base.SCO_DET_ID_NAMESPACE, name)}"
    return f"{stix_class._type}--{uuid.uuid4()}"


class StixObjects:
    """
    Helper class for collecting unique STIX instances (by STIX ID)
    """
    def __init__(self, objects: Union[List, None]=None):
        self._container = {}
        if objects:
            self.extend(objects)

    def append(self, item):
        item_id = item.id
        try:
            if item_id not in self._container:
                self._container[item_id] = item
        except AttributeError:
            raise

    def extend(self, __iterable):
        for i in __iterable:
            self.append(i)

    def get(self):
        return self._container.values()

    def __bool__(self):
        return bool(self._container)


author_name = "Intel 471 Inc."
author_identity = Identity(
    id=generate_id(Identity, name=author_name.lower(), identity_class="organization"),
    name=author_name,
    identity_class="organization",
    created=datetime.datetime(2022, 1, 1),
    modified=datetime.datetime(2022, 1, 1),
)



import yaml
from stix2 import Bundle, Vulnerability, ExternalReference, Software, Relationship

from .common import BaseMapper, StixMapper
from .. import author_identity, generate_id, StixObjects
from ..constants import MARKING


@StixMapper.register("cves", lambda x: "cveReportsTotalCount" in x)
@StixMapper.register("cve", lambda x: "cve_report" in x.get("data", {}))
class CveMapper(BaseMapper):
    def map(self, source: dict) -> Bundle:
        container = StixObjects()
        items = (
            source.get("cveReports") or []
            if "cveReportsTotalCount" in source
            else [source]
        )
        for item in items:
            uid = item["uid"]
            name = item["data"]["cve_report"]["name"]
            summary = item["data"]["cve_report"]["summary"]
            underground_activity_summary = item["data"]["cve_report"][
                "underground_activity_summary"
            ]
            counter_measures = item["data"]["cve_report"]["counter_measures"]
            extras = yaml.dump(
                {
                    k: v
                    for k, v in item["data"]["cve_report"].items()
                    if k
                    in (
                        "patch_status",
                        "interest_level",
                        "activity_location",
                        "exploit_status",
                    )
                }
            )
            description = f"{summary}\n\n{underground_activity_summary}\n\n{counter_measures}\n\n### Properties\n\n```yaml\n{extras}```"
            cvss3_score = (item["data"]["cve_report"].get("cvss_score") or {}).get("v3")
            external_references = []
            for link_type, key in (
                ("Titan URL", "titan_links"),
                ("PoC", "poc_links"),
                ("Patch", "patch_links"),
            ):
                for link in item["data"]["cve_report"].get(key) or []:
                    external_reference = ExternalReference(
                        source_name=f"[{link_type}] {link['title']}", url=link["url"]
                    )
                    external_references.append(external_reference)

            custom_properties = {"x_intel471_com_uid": uid}
            if cvss3_score:
                custom_properties["x_opencti_base_score"] = cvss3_score
            labels = self.get_girs_labels((item.get("classification") or {}).get("intel_requirements") or [])
            vulnerability = Vulnerability(
                id=generate_id(Vulnerability, name=name.strip().lower()),
                name=name,
                description=description,
                created_by_ref=author_identity,
                external_references=external_references,
                object_marking_refs=[MARKING],
                custom_properties=custom_properties,
                labels=labels
            )
            software = Software(
                name=item["data"]["cve_report"]["product_name"],
                vendor=item["data"]["cve_report"]["vendor_name"])
            rel = Relationship(software, "has", vulnerability, created_by_ref=author_identity)
            container.extend([vulnerability, software, rel, author_identity, MARKING])
        if container:
            bundle = Bundle(*container.get(), allow_custom=True)
            return bundle



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from titan_client.api.actors_api import ActorsApi
from titan_client.api.alerts_api import AlertsApi
from titan_client.api.credentials_api import CredentialsApi
from titan_client.api.data_leak_blogs_api import DataLeakBlogsApi
from titan_client.api.entities_api import EntitiesApi
from titan_client.api.events_api import EventsApi
from titan_client.api.forums_api import ForumsApi
from titan_client.api.girs_api import GIRsApi
from titan_client.api.global_search_api import GlobalSearchApi
from titan_client.api.iocs_api import IOCsApi
from titan_client.api.indicators_api import IndicatorsApi
from titan_client.api.malware_families_api import MalwareFamiliesApi
from titan_client.api.marketplaces_api import MarketplacesApi
from titan_client.api.messaging_services_api import MessagingServicesApi
from titan_client.api.news_api import NewsApi
from titan_client.api.reports_api import ReportsApi
from titan_client.api.tags_api import TagsApi
from titan_client.api.vulnerabilities_api import VulnerabilitiesApi
from titan_client.api.watchers_api import WatchersApi
from titan_client.api.yara_api import YARAApi



from titan_client.titan_stix.mappers.cves import CveMapper
from titan_client.titan_stix.mappers.indicators import IndicatorsMapper
from titan_client.titan_stix.mappers.iocs import IOCMapper
from titan_client.titan_stix.mappers.reports import ReportMapper
from titan_client.titan_stix.mappers.yara import YaraMapper



import datetime
import logging
from typing import Union

from pytz import UTC
from stix2 import Bundle, Indicator, Report, DomainName, URL, Relationship

from .. import author_identity, generate_id, StixObjects
from ..constants import MARKING
from ..patterning import create_domain_pattern, create_url_pattern, create_ipv4_pattern, create_file_pattern
from ..sco import map_domain, map_url, map_ipv4, map_file
from .reports import ReportMapper
from .common import StixMapper, BaseMapper, MappingConfig

log = logging.getLogger(__name__)


@StixMapper.register("iocs", lambda x: "iocTotalCount" in x)
class IOCMapper(BaseMapper):

    mapping_configs = {
        "MaliciousURL": MappingConfig(
            patterning_mapper=create_url_pattern,
            entities_mapper=map_url,
            kwargs_extractor=lambda i: {"value": i["value"]},
            name_extractor=lambda i: i["value"],
            opencti_type="Url"
        ),
        "MaliciousDomain": MappingConfig(
            patterning_mapper=create_domain_pattern,
            entities_mapper=map_domain,
            kwargs_extractor=lambda i: {"value": i["value"].split("://")[-1]},
            name_extractor=lambda i: i["value"].split("://")[-1],
            opencti_type="Domain-Name"
        ),
        "IPAddress": MappingConfig(
            patterning_mapper=create_ipv4_pattern,
            entities_mapper=map_ipv4,
            kwargs_extractor=lambda i: {"value": i["value"]},
            name_extractor=lambda i: i["value"],
            opencti_type="IPv4-Addr"
        ),
        "MD5": MappingConfig(
            patterning_mapper=create_file_pattern,
            entities_mapper=map_file,
            kwargs_extractor=lambda i: {"md5": i["value"]},
            name_extractor=lambda i: i["value"],
            opencti_type="StixFile"
        ),
        "SHA1": MappingConfig(
            patterning_mapper=create_file_pattern,
            entities_mapper=map_file,
            kwargs_extractor=lambda i: {"sha1": i["value"]},
            name_extractor=lambda i: i["value"],
            opencti_type="StixFile"
        ),
        "SHA256": MappingConfig(
            patterning_mapper=create_file_pattern,
            entities_mapper=map_file,
            kwargs_extractor=lambda i: {"sha256": i["value"]},
            name_extractor=lambda i: i["value"],
            opencti_type="StixFile"
        )
    }

    def map(self, source: dict) -> Bundle:
        container = StixObjects()
        report_mapper = ReportMapper(self.settings)
        items = source.get("iocs") or [] if "iocTotalCount" in source else [source]
        for item in items:
            ioc_type = item["type"]
            mapping_config = self.mapping_configs.get(ioc_type)
            if not mapping_config:
                continue
            report_sources = item["links"].get("reports") or []
            valid_from = datetime.datetime.fromtimestamp(item["activeFrom"] / 1000, UTC)
            valid_until = datetime.datetime.fromtimestamp(
                item["activeTill"] / 1000, UTC
            )
            if valid_from == valid_until:
                valid_until = None

            name = mapping_config.name_extractor(item)
            kwargs = mapping_config.kwargs_extractor(item)
            stix_pattern = mapping_config.patterning_mapper(**kwargs)
            observable = self.map_entity(item)
            indicator = Indicator(
                id=generate_id(Indicator, pattern=stix_pattern),
                name=name,
                pattern_type="stix",
                pattern=stix_pattern,
                indicator_types=["malicious-activity"],
                valid_from=valid_from,
                valid_until=valid_until,
                created_by_ref=author_identity,
                object_marking_refs=[MARKING],
                custom_properties={"x_opencti_main_observable_type": mapping_config.opencti_type}
            )
            r1 = Relationship(
                indicator, "based-on", observable, created_by_ref=author_identity
            )
            container.extend([indicator, observable, r1, author_identity, MARKING])
            for stix_object in self._map_reports(
                report_mapper, report_sources, indicator, observable, r1).get():
                if isinstance(stix_object, Report):
                    if already_mapped_reports := [i for i in container.get() if i.id == stix_object.id]:
                        already_mapped_report = already_mapped_reports[0]
                        already_mapped_report.object_refs.extend(stix_object.object_refs)
                container.append(stix_object)
        if container:
            bundle = Bundle(*container.get(), allow_custom=True)
            return bundle

    def map_entity(self, source: dict):
        mapping_config = self.mapping_configs.get(source["type"])
        kwargs = mapping_config.kwargs_extractor(source)
        return mapping_config.entities_mapper(**kwargs)

    @staticmethod
    def _map_reports(
        report_mapper,
        report_sources: list,
        indicator: Indicator,
        observable: Union[URL, DomainName],
        relationship: Relationship
    ) -> StixObjects:
        stix_objects = StixObjects()
        for report_source in report_sources:
            report_etc: StixObjects = report_mapper.map_report_ioc(
                report_source,
                object_refs=StixObjects([indicator, observable, relationship]))
            stix_objects.extend(report_etc.get())
        return stix_objects



import datetime

import yaml
from pytz import UTC
from stix2 import Indicator, Bundle, Relationship, KillChainPhase, TLP_AMBER

from .. import author_identity, generate_id, StixObjects
from .common import StixMapper, BaseMapper, MappingConfig
from ..patterning import create_url_pattern, create_ipv4_pattern, create_file_pattern
from ..sco import map_url, map_ipv4, map_file
from ..sdo import map_malware


@StixMapper.register("indicators", lambda x: "indicatorTotalCount" in x)
class IndicatorsMapper(BaseMapper):

    mapping_configs = {
        "url": MappingConfig(
            patterning_mapper=create_url_pattern,
            entities_mapper=map_url,
            kwargs_extractor=lambda i: {"value": i["data"]["indicator_data"]["url"]},
            name_extractor=lambda i: i["data"]["indicator_data"]["url"],
            opencti_type="Url"
        ),
        "ipv4": MappingConfig(
            patterning_mapper=create_ipv4_pattern,
            entities_mapper=map_ipv4,
            kwargs_extractor=lambda i: {
                "value": i["data"]["indicator_data"]["address"]
            },
            name_extractor=lambda i: i["data"]["indicator_data"]["address"],
            opencti_type="IPv4-Addr"
        ),
        "file": MappingConfig(
            patterning_mapper=create_file_pattern,
            entities_mapper=map_file,
            kwargs_extractor=lambda i: {
                "md5": i["data"]["indicator_data"]["file"]["md5"],
                "sha1": i["data"]["indicator_data"]["file"]["sha1"],
                "sha256": i["data"]["indicator_data"]["file"]["sha256"],
            },
            name_extractor=lambda i: i["data"]["indicator_data"]["file"]["sha256"],
            opencti_type="StixFile"
        ),
    }

    def map(self, source: dict) -> Bundle:
        container = StixObjects()
        items = (
            source.get("indicators") or []
            if "indicatorTotalCount" in source
            else [source]
        )
        for item in items:
            indicator_type = item["data"]["indicator_type"]
            mapping_config = self.mapping_configs.get(indicator_type)
            if not mapping_config:
                continue
            malware_family_name = item["data"]["threat"]["data"]["family"]
            valid_from = datetime.datetime.fromtimestamp(
                item["activity"]["first"] / 1000, UTC
            )
            valid_until = datetime.datetime.fromtimestamp(
                item["data"]["expiration"] / 1000, UTC
            )
            tactics = self.map_tactic(item["data"]["mitre_tactics"])
            confidence = self.map_confidence(item["data"]["confidence"])
            name = mapping_config.name_extractor(item)
            description = item["data"]["context"]["description"]
            kwargs = mapping_config.kwargs_extractor(item)
            stix_pattern = mapping_config.patterning_mapper(**kwargs)
            observable = mapping_config.entities_mapper(**kwargs)
            malware = map_malware(malware_family_name)
            custom_properties = {"x_opencti_main_observable_type": mapping_config.opencti_type}
            if self.settings.ioc_opencti_score:
                custom_properties.update({"x_opencti_score": self.settings.ioc_opencti_score})
            labels = [malware_family_name]
            labels.extend(self.get_girs_labels(item["data"]["intel_requirements"]))
            indicator = Indicator(
                id=generate_id(Indicator, pattern=stix_pattern),
                pattern_type="stix",
                pattern=stix_pattern,
                indicator_types=["malicious-activity"],
                valid_from=valid_from,
                valid_until=valid_until,
                created_by_ref=author_identity,
                object_marking_refs=[TLP_AMBER],
                name=name,
                description=description,
                labels=labels,
                confidence=confidence,
                kill_chain_phases=[
                    KillChainPhase(kill_chain_name="mitre-attack", phase_name=tactics)
                ],
                custom_properties=custom_properties
            )
            r1 = Relationship(
                indicator, "indicates", malware, created_by_ref=author_identity
            )
            r2 = Relationship(
                indicator, "based-on", observable, created_by_ref=author_identity
            )
            for stix_object in [
                malware,
                indicator,
                observable,
                r1,
                r2,
                author_identity,
                TLP_AMBER,
            ]:
                container.append(stix_object)
        if container:
            bundle = Bundle(*container.get(), allow_custom=True)
            return bundle



import datetime

import yaml
from pytz import UTC
from stix2 import Indicator, Bundle, Relationship, TLP_AMBER
from .common import StixMapper, BaseMapper
from .. import author_identity, generate_id, StixObjects
from ..sdo import map_malware


@StixMapper.register("yara", lambda x: "yaraTotalCount" in x)
class YaraMapper(BaseMapper):
    def map(self, source: dict) -> Bundle:
        container = StixObjects()
        items = source.get("yaras") or [] if "yaraTotalCount" in source else [source]
        for item in items:
            yara_signature = item["data"]["yara_data"]["signature"]
            malware_family_name = item["data"]["threat"]["data"]["family"]
            valid_from = datetime.datetime.fromtimestamp(
                item["activity"]["first"] / 1000, UTC
            )
            confidence = self.map_confidence(item["data"]["confidence"])
            labels = [malware_family_name]
            labels.extend(self.get_girs_labels(item["data"]["intel_requirements"]))
            malware = map_malware(malware_family_name)
            indicator = Indicator(
                id=generate_id(Indicator, pattern=yara_signature),
                pattern_type="yara",
                pattern=yara_signature,
                indicator_types=["malicious-activity"],
                valid_from=valid_from,
                created_by_ref=author_identity,
                object_marking_refs=[TLP_AMBER],
                labels=labels,
                confidence=confidence,
            )
            relationship = Relationship(
                indicator, "indicates", malware, created_by_ref=author_identity
            )
            for stix_object in [
                malware,
                indicator,
                relationship,
                author_identity,
                TLP_AMBER,
            ]:
                container.append(stix_object)
        if container:
            bundle = Bundle(*container.get(), allow_custom=True)
            return bundle



import json
import os

import pytest
from stix2 import DomainName

from titan_client.titan_stix import STIXMapperSettings, StixObjects
from titan_client.titan_stix.mappers import ReportMapper
from titan_client.titan_stix.mappers.common import StixMapper
from titan_client.titan_stix.mappers.entities import EntitiesMapper
from titan_client.titan_stix.mappers.reports import ReportType

from .conftest import PREFIX, read_fixture
from mock import MagicMock

os.environ['I471_TITAN_CLIENT_CACHE_TTL'] = '0'

fixtures = {
    'test_indicators': ("indicators_input.json", "indicators_stix.json"),
    'test_iocs': ("iocs_input.json", "iocs_stix.json"),
    'test_yara': ("yara_input.json", "yara_stix.json"),
    'test_cves': ("cves_input.json", "cves_stix.json"),
    'test_cvesx': ("iocs_with_reports_input.json", "reports_from_iocs_stix.json"),
    'test_report_breach_alert': ("report_breach_alert_input.json", "report_breach_alert_stix.json"),
    'test_report_fintel': ("report_fintel_input.json", "report_fintel_stix.json"),
    'test_report_inforep': ("report_inforep_input.json", "report_inforep_stix.json"),
    'test_report_spot': ("report_spot_input.json", "report_spot_stix.json"),
    'test_report_malware': ("report_malware_input.json", "report_malware_stix.json"),
}


def strip_random_values(bundle: dict) -> dict:
    bundle["id"] = None
    remove_id_types = ('relationship', 'threat-actor', 'identity')
    for i1, o in enumerate(bundle["objects"]):
        bundle["objects"][i1]["created"] = None
        bundle["objects"][i1]["modified"] = None
        if o["type"] in remove_id_types:
            bundle["objects"][i1]["id"] = None
        for i2, object_ref_id in enumerate(o.get("object_refs", [])):
            if object_ref_id.split("--")[0] in remove_id_types:
                bundle["objects"][i1]["object_refs"][i2] = None

    return bundle


@pytest.mark.parametrize('fixtures', fixtures.values(), ids=fixtures.keys())
def test_stix_mappers(fixtures):
    in_fixture, out_fixture = fixtures
    api_response = read_fixture(f'{PREFIX}/fixtures/{in_fixture}')
    expected_result = read_fixture(f'{PREFIX}/fixtures/{out_fixture}')

    gir_mock = MagicMock(name="Gir mock")
    gir_mock.data.gir.path, gir_mock.data.gir.name = "1.1.1", "Lorem"
    api_response_mock = MagicMock(name="API response")
    api_response_mock.girs = [gir_mock]
    api_instance_mock = MagicMock(name="API instance")
    api_instance_mock.girs_get.return_value = api_response_mock
    mock_titan_client = MagicMock(name="titan_client")
    mock_titan_client.GIRsApi.return_value = api_instance_mock
    mock_api_client = MagicMock()

    mapper = StixMapper(STIXMapperSettings(
        titan_client=mock_titan_client, api_client=mock_api_client, report_full_content=False
    ))
    result = mapper.map(api_response)
    expected = strip_random_values(expected_result)
    assert expected == strip_random_values(json.loads(result.serialize()))


def test_report_from_ioc():
    mock_domain = DomainName(value="foo.bar")
    mapper = ReportMapper(STIXMapperSettings())
    result = mapper.map_report_ioc({
        "subject": "New malware released",
        "released": 1679321907000,
        "portalReportUrl": "https://foo.bar/fintel/123",
        "uid": "1fffffffffffffffffffffffffffffff",
        "admiraltyCode": "A1",
        "dateOfInformation": 1678060800000
    }, StixObjects([mock_domain]))
    report_serialized = json.loads(list(result.get())[-1].serialize())
    assert report_serialized["name"] == "New malware released"
    assert report_serialized["description"] == "New malware released"
    assert report_serialized["report_types"] == ["fintel"]
    assert report_serialized["confidence"] == 90
    assert getattr(report_serialized, "content", None) is None
    assert getattr(report_serialized, "labels", None) is None


def test_full_report_from_ioc():
    api_cls = "ReportsApi"
    method_name = "reports_uid_get"
    api_response = {
        "uid": "1fffffffffffffffffffffffffffffff",
        "documentFamily": "FINTEL",
        "documentType": "ACTOR_PROFILE",
        "subject": "New malware released (fromAPI)",
        "admiraltyCode": "A1",
        "created": 1679321907000,
        "dateOfInformation": 1678060800000,
        "rawText": "<h2>Foo</h2><p>New malware <strong>Foobar</strong> released!</p><h2>Bar</h2>",
        "classification": {"intelRequirements": ["1.1"]}
        }

    api_response_mock = MagicMock(name="API response")
    api_response_mock.to_dict.return_value = api_response
    api_instance_mock = MagicMock(name="API instance")
    getattr(api_instance_mock, method_name).return_value = api_response_mock
    mock_titan_client = MagicMock(name="titan_client")
    getattr(mock_titan_client, api_cls).return_value = api_instance_mock
    mock_api_client = MagicMock()
    mock_domain = DomainName(value="foo.bar")

    mapper = ReportMapper(STIXMapperSettings(mock_titan_client, mock_api_client, report_full_content=True))
    result = mapper.map_report_ioc({
        "subject": "New malware released",
        "released": 1679321907000,
        "portalReportUrl": "https://foo.bar/fintel/123",
        "uid": "1fffffffffffffffffffffffffffffff",
        "admiraltyCode": "A1",
        "dateOfInformation": 1678060800000
    }, StixObjects([mock_domain]))
    report_serialized = json.loads(list(result.get())[-1].serialize())
    assert report_serialized["name"] == "New malware released (fromAPI)"
    assert report_serialized["description"] == "New malware Foobar released!"
    assert report_serialized["report_types"] == ["fintel", "actor_profile"]
    assert report_serialized["confidence"] == 90
    assert report_serialized["content"] == api_response['rawText']
    assert report_serialized["labels"] == ["Intel 471 - GIR 1.1"]


def test_full_report_from_reports_api():
    api_cls = "ReportsApi"
    method_name = "reports_uid_get"
    api_response = {
        "uid": "1fffffffffffffffffffffffffffffff",
        "documentFamily": "INFOREP",
        "documentType": "INFOREP",
        "subject": "New malware released (fromAPI)",
        "admiraltyCode": "A1",
        "created": 1679321907000,
        "dateOfInformation": 1678060800000,
        "classification": {"intelRequirements": ["1.1"]},
        "entities": [{"type": "ActorDomain", "value": "foo.bar"}],
        "executiveSummary": "This report examines the cybercriminal underground",
        "rawText": "<h2>Foo</h2><p>New malware <strong>Foobar</strong> released!</p><h2>Bar</h2>",
        "researcherComments": "<h3>Actor and information assessment</h3><p>The actor joined the XYZ forum"
                              "<figure class=\"image image_resized width52\"><img src=\"data:image/png;base64,iVBORw0KGgoAAAANSU</figure></p>",
        }

    api_response_mock = MagicMock(name="API response")
    api_response_mock.to_dict.return_value = api_response
    api_instance_mock = MagicMock(name="API instance")
    getattr(api_instance_mock, method_name).return_value = api_response_mock
    mock_titan_client = MagicMock(name="titan_client")
    getattr(mock_titan_client, api_cls).return_value = api_instance_mock
    mock_api_client = MagicMock()

    mapper = ReportMapper(STIXMapperSettings(mock_titan_client, mock_api_client, report_full_content=True))
    bundle = mapper.map({
        "documentFamily": "FINTEL",
        "subject": "New malware released",
        "released": 1679321907000,
        "uid": "1fffffffffffffffffffffffffffffff",
        "admiraltyCode": "A1",
        "dateOfInformation": 1678060800000
    })
    report = bundle.objects[-1]
    assert report.name == "New malware released (fromAPI)"
    assert report.description == "This report examines the cybercriminal underground"
    assert report.report_types == ["inforep"]
    assert report.confidence == 90
    assert report.labels == ["Intel 471 - GIR 1.1"]
    assert report.content == ('<h1>Executive Summary</h1>\n'
                              'This report examines the cybercriminal underground\n'
                              '<h1>Researcher Comments</h1>\n'
                              '<h3>Actor and information assessment</h3>'
                              '<p>The actor joined the XYZ forum<figure class="image image_resized width52"></p>\n'
                              '<h1>Raw Text</h1>\n'
                              '<h2>Foo</h2>'
                              '<p>New malware <strong>Foobar</strong> released!</p><h2>Bar</h2>')


@pytest.mark.parametrize("source,expected_values", (
        # /iocs endpoint
        ({"type": "MaliciousURL", "value": "https://acme.com"}, {"type": "url", "value": "https://acme.com"}),
        ({"type": "URL", "value": "https://acme.com"}, {"type": "url", "value": "https://acme.com"}),
        ({"type": "MaliciousDomain", "value": "acme.com"}, {"type": "domain-name", "value": "acme.com"}),
        ({"type": "IPAddress", "value": "10.0.0.1"}, {"type": "ipv4-addr", "value": "10.0.0.1"}),
        ({"type": "IPv4Prefix", "value": "10.0.0.1/24"}, {"type": "ipv4-addr", "value": "10.0.0.1/24"}),
        ({"type": "IPv6Prefix", "value": "2a09:7c44::/32"}, {"type": "ipv6-addr", "value": "2a09:7c44::/32"}),
        ({"type": "AutonomousSystem", "value": "AS123456"}, {"type": "autonomous-system", "number": 123456}),
        ({"type": "MD5", "value": "acb9cf2560602aa023e9933b959974d1"}, {"type": "file", "hashes": {"MD5": "acb9cf2560602aa023e9933b959974d1"}}),
        ({"type": "SHA1", "value": "ae9de44e5f23758ffb6f4fa28065c6122c2e4467"}, {"type": "file", "hashes": {"SHA-1": "ae9de44e5f23758ffb6f4fa28065c6122c2e4467"}}),
        ({"type": "SHA256", "value": "890a0bfab48d0b93da8f7617b2e65621e8d7f8c93a854fa8596232d9bcb1b39e"}, {"type": "file", "hashes": {"SHA-256": "890a0bfab48d0b93da8f7617b2e65621e8d7f8c93a854fa8596232d9bcb1b39e"}}),
        ({"type": "FileName", "value": "acmecorp.exe"}, {"type": "file", "name": "acmecorp.exe"}),
        # /entities endpoint
        ({"type": "ActorDomain", "value": "acme.com"}, {"type": "domain-name", "value": "acme.com"}),
        ({"type": "Handle", "value": "acme"}, {"type": "threat-actor", "name": "acme"}),
        ({"type": "ActorOtherWebsite", "value": "https://acme.com"}, {"type": "url", "value": "https://acme.com"}),
        ({"type": "BitcoinAddress", "value": "1HUu6K9sFvN1cGV"}, {"type": "cryptocurrency-wallet", "value": "1HUu6K9sFvN1cGV"}),
        ({"type": "OtherCryptoCurrencies", "value": "1HUu6K9sFvN1cGV"}, {"type": "cryptocurrency-wallet", "value": "1HUu6K9sFvN1cGV"}),
        ({"type": "EmailAddress", "value": "changeme@acme.com"}, {"type": "email-addr", "value": "changeme@acme.com"}),
        ({"type": "AIM", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "AIM"}),
        ({"type": "Discord", "value": "https://discord.gg/2xx2xx2"}, {"type": "user-account", "user_id": "2xx2xx2", "account_type": "Discord"}),
        ({"type": "Facebook", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Facebook"}),
        ({"type": "GitHub", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "GitHub"}),
        ({"type": "ICQ", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "ICQ"}),
        ({"type": "Instagram", "value": "instagram.com/changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Instagram"}),
        ({"type": "Jabber", "value": "changeme@jabber.de"}, {"type": "user-account", "user_id": "changeme@jabber.de", "account_type": "Jabber"}),
        ({"type": "LinkedIn", "value": "https://www.linkedin.com/in/iamweasel-58312b1a6/"}, {"type": "user-account", "user_id": "iamweasel-58312b1a6", "account_type": "LinkedIn"}),
        ({"type": "MSN", "value": "changeme@acme.com"}, {"type": "user-account", "user_id": "changeme@acme.com", "account_type": "MSN"}),
        ({"type": "MoiMir", "value": "my.mail.ru/mail/changeme/"}, {"type": "user-account", "user_id": "changeme", "account_type": "MoiMir"}),
        ({"type": "Odnoklassniki", "value": "ok.ru/profile/11111800259"}, {"type": "user-account", "user_id": "11111800259", "account_type": "Odnoklassniki"}),
        ({"type": "QQ", "value": "111159547"}, {"type": "user-account", "user_id": "111159547", "account_type": "QQ"}),
        ({"type": "Skype", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Skype"}),
        ({"type": "Telegram", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Telegram"}),
        ({"type": "TOX", "value": "2B41B398739E6BECE4E93EAFA0C665"}, {"type": "user-account", "user_id": "2B41B398739E6BECE4E93EAFA0C665", "account_type": "TOX"}),
        ({"type": "Twitter", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Twitter"}),
        ({"type": "VK", "value": "vk.com/changeme_2"}, {"type": "user-account", "user_id": "changeme_2", "account_type": "VK"}),
        ({"type": "WeChat", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "WeChat"}),
        ({"type": "Wickr", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Wickr"}),
        ({"type": "YahooIM", "value": "changeme@yahoo.com"}, {"type": "user-account", "user_id": "changeme@yahoo.com", "account_type": "YahooIM"}),
        ({"type": "PerfectMoneyID", "value": "U1111111"}, {"type": "user-account", "user_id": "U1111111", "account_type": "PerfectMoneyID"}),
        ({"type": "QiwiWallet", "value": "11110519386"}, {"type": "user-account", "user_id": "11110519386", "account_type": "QiwiWallet"}),
        ({"type": "WebMoneyID", "value": "111112935229"}, {"type": "user-account", "user_id": "111112935229", "account_type": "WebMoneyID"}),
        ({"type": "WebMoneyPurse", "value": "Z111144083730"}, {"type": "user-account", "user_id": "Z111144083730", "account_type": "WebMoneyPurse"}),
        ({"type": "YandexMoney", "value": "111113131482342"}, {"type": "user-account", "user_id": "111113131482342", "account_type": "YandexMoney"}),
        ({"type": "Phone", "value": "79874172111"}, {"type": "user-account", "user_id": "79874172111", "account_type": "Phone"}),
        ({"type": "CveID", "value": "CVE-2024-23113"}, {"type": "vulnerability", "name": "CVE-2024-23113"}),
        # Invalid stuff
        ({"type": "Handle", "value": "a"}, None),
        ({"type": "MD5", "value": "invalidMd5"}, None)
))
def test_observable_mapper(source, expected_values):
    mapper = EntitiesMapper()
    sco = mapper.map(**source)
    if expected_values is None:
        assert sco is expected_values
    else:
        for key, value in expected_values.items():
            assert getattr(sco, key) == value


@pytest.mark.parametrize("report_type,source,expected_values", (
    (ReportType.FINTEL.value, {"uid": "ab1", "documentFamily": "FINTEL", "sources": [
        {"type": "External Link", "title": "ACME corp news", "url": "https://acme.corp/123", "index": "1"}]},
     {"source_name": "External Link - ACME corp news", "url": "https://acme.corp/123"}),
    (ReportType.FINTEL.value, {"uid": "ab1", "documentFamily": "FINTEL", "sources": [
        {"type": "External Link", "title": "Titan Information Report", "url": "https://titan.intel471.com/report/inforep/487a8", "index": "1"}]},
     {}),
    (ReportType.INFOREP.value, {"uid": "ab1", "documentFamily": "INFOREP", "sources": [
        {"type": "Forum Post", "title": "[SOURCE CODE] HexSec | Android RAT",
         "url": "https://titan.intel471.com/post_thread/9cacd56", "index": "1"}]},
     {"source_name": "Forum Post - [SOURCE CODE] HexSec | Android RAT", "url": "https://titan.intel471.com/post_thread/9cacd56"}),
    (ReportType.INFOREP.value, {"uid": "ab1", "documentFamily": "INFOREP", "sources": [
        {"type": "Forum Post", "title": "[SOURCE CODE] HexSec | Android RAT",
         "url": "https://titan.intel471.com/post_thread/9cacd56", "index": "1"}]},
     {"source_name": "Forum Post - [SOURCE CODE] HexSec | Android RAT",
      "url": "https://titan.intel471.com/post_thread/9cacd56"}),
    (ReportType.BREACH_ALERT.value, {"uid": "ab1", "data": {"breach_alert": {"sources": [
        {"type": "internal", "source_type": "Forum Thread", "title": "acmesystems",
         "url": "https://titan.intel471.com/post_thread/2984", "index": "1"}]}}},
     {"source_name": "Forum Thread internal - acmesystems",
      "url": "https://titan.intel471.com/post_thread/2984"}),
    (ReportType.SPOTREP.value, {"uid": "ab1", "data": {"spot_report": {"spot_report_data": {"links": [
        {"type": "internal", "title": "Forum thread",
         "url": "https://titan.intel471.com/post_thread/2984"}]}}}},
     {"source_name": "internal - Forum thread",
      "url": "https://titan.intel471.com/post_thread/2984"}),
))
def test_map_reports_external_references(report_type, source, expected_values):
    mapper = ReportMapper(STIXMapperSettings())
    external_refs = mapper._get_external_references(source)
    external_ref_0 = json.loads(external_refs[0].serialize())
    assert external_ref_0 == {"source_name": "Titan URL", "url": f"https://titan.intel471.com/report/{report_type}/ab1"}
    if expected_values:
        external_ref_1 = json.loads(external_refs[1].serialize())
        assert external_ref_1 == expected_values


@pytest.mark.parametrize("report_type,source", (
    (ReportType.FINTEL.value, {"uid": "ab1", "documentFamily": "FINTEL", "entities": [{"type": "MalwareFamily", "value": "acme"}, {"type": "IPAddress", "value": "0.0.0.1"}]}),
    (ReportType.FINTEL.value, {"uid": "ab1", "documentFamily": "INFOREP", "entities": [{"type": "MalwareFamily", "value": "acme"}]}),
    (ReportType.BREACH_ALERT.value, {"uid": "ab1", "data": {"breach_alert": {}, "entities": [{"type": "MalwareFamily", "value": "acme"}]}}),
    (ReportType.SPOTREP.value, {"uid": "ab1", "data": {"spot_report": {}, "entities": [{"type": "MalwareFamily", "value": "acme"}]}}),
    (ReportType.MALWARE.value, {"uid": "ab1", "data": {"malware_report_data": {}, "threat": {"data": {"family": "acme"}}}}),
))
def test_map_reports_entities(report_type, source):
    mapper = ReportMapper(STIXMapperSettings())
    entities = mapper._get_entities(source)
    entity = json.loads(list(entities.get())[0].serialize())
    assert entity["type"] == "malware"
    assert entity["name"] == "acme"
    malware_families_names = mapper._get_malware_families_names(entities)
    assert malware_families_names == ["acme"]


@pytest.mark.parametrize("report_type,source", (
    (ReportType.FINTEL.value, {"uid": "ab1", "documentFamily": "FINTEL", "victims": [
        {"name": "ACME corp", "urls": ["https://acme.corp"]}]}),
    (ReportType.INFOREP.value, {"uid": "ab1", "documentFamily": "INFOREP", "victims": [
        {"name": "ACME corp", "urls": ["https://acme.corp"]}]}),
    (ReportType.BREACH_ALERT.value, {"uid": "ab1", "data": {"breach_alert": {"victim": {
            "country": "United States",
            "industries": [
                {
                    "industry": "Financial and investment consulting industry",
                    "sector": "Professional services and consulting sector"
                }
            ],
            "name": "ACME corp",
            "region": "North America",
            "revenue": "US $100 million",
            "urls": [
                "https://acme.corp"
            ]
        }}}}),
    (ReportType.SPOTREP.value, {"uid": "ab1", "data": {"spot_report": {"spot_report_data": {"victims": [
        {"name": "ACME corp", "urls": ["https://acme.corp"]}]}}}}),
))
def test_map_reports_victims(report_type, source):
    mapper = ReportMapper(STIXMapperSettings())
    victims = list(mapper._get_victims(source).get())
    assert victims[0].type == "identity"
    assert victims[0].identity_class == "organization"
    assert victims[0].name == "ACME corp"
    assert victims[0].contact_information == "https://acme.corp"


@pytest.mark.parametrize("report_type,source", (
        (ReportType.FINTEL, {"uid": "123", "documentFamily": "FINTEL", "classification": {"intelRequirements": ["1.1.1", "1.2.2"]}}),
        (ReportType.INFOREP, {"uid": "123", "documentFamily": "INFOREP", "classification": {"intelRequirements": ["1.1.1", "1.2.2"]}}),
        (ReportType.SPOTREP, {"uid": "123", "data": {"spot_report": {"spot_report_data":{"intel_requirements": ["1.1.1", "1.2.2"]}}}}),
        (ReportType.MALWARE, {"uid": "123", "data": {"malware_report_data": {}}, "classification": {"intelRequirements": ["1.1.1", "1.2.2"]}}),
        (ReportType.BREACH_ALERT, {"uid": "123", "data": {"breach_alert":{"intel_requirements": ["1.1.1", "1.2.2"]}}}),
))
def test_reports_gir_labels(report_type, source):
    mapper = ReportMapper(STIXMapperSettings())
    girs_labels = mapper._get_girs_labels(source)
    assert girs_labels == ["Intel 471 - GIR 1.1.1", "Intel 471 - GIR 1.2.2"]



import logging
import re
from collections import ChainMap
from dataclasses import dataclass
from typing import Union, Callable

from stix2.exceptions import InvalidValueError

from titan_client.titan_stix.sco import map_url, map_domain, map_ipv4, map_ipv6, map_email_address, \
    map_autonomous_system, map_file_hash, map_filename, map_crypto_wallet, map_user_account
from titan_client.titan_stix.sdo import map_malware, map_threat_actor, map_vulnerability


log = logging.getLogger(__name__)


@dataclass(frozen=True)
class MapperConfig:
    mapper: Callable
    src_types: tuple
    value_extractor: Union[Callable, None] = None


class EntitiesMapper:
    """
    Map entities attached to the reports under `.entities` key and entities
    present in `/entities` and `/iocs` endpoints.
    """
    def __init__(self):
        self.mapper_configs = [
            MapperConfig(map_url, ("MaliciousURL", "ActorOtherWebsite", "URL")),
            MapperConfig(map_domain, ("MaliciousDomain", "ActorDomain")),
            MapperConfig(map_ipv4, ("IPAddress", "IPv4Prefix")),
            MapperConfig(map_ipv6, ("IPv6Prefix", )),
            MapperConfig(map_email_address, ("EmailAddress", )),
            MapperConfig(map_autonomous_system, ("AutonomousSystem",)),
            MapperConfig(map_file_hash, ("MD5", "SHA1", "SHA256",)),
            MapperConfig(map_filename, ("FileName",)),
            MapperConfig(map_crypto_wallet, ("BitcoinAddress", "OtherCryptoCurrencies")),
            MapperConfig(map_threat_actor, ("Handle",)),
            MapperConfig(map_malware, ("MalwareFamily",)),
            MapperConfig(map_vulnerability, ("CveID",)),
            MapperConfig(map_user_account, (
                "AIM", "Discord", "Facebook", "GitHub", "ICQ", "Instagram", "Jabber", "LinkedIn", "MSN", "MoiMir",
                "Odnoklassniki", "QQ", "Skype", "Telegram", "TOX", "Tox", "Twitter", "VK", "WeChat", "Wickr", "YahooIM",
                "PerfectMoneyID", "QiwiWallet", "WebMoneyID", "WebMoneyPurse", "YandexMoney", "Phone"),
                         lambda x: re.sub(r"^.*/", "", x.strip("/"))),
        ]
        self.type2config = dict(ChainMap(*[{st: mc for st in mc.src_types} for mc in self.mapper_configs]))

    def map(self, type: str, value: str, *args, **kwargs):  # noqa need `type` here as it comes from the API
        if len(value) < 2:
            log.info(f"Skipping entity. Value too short. type=`{type}` value=`{value}`")
            return
        if mapper_config := self.type2config.get(type):
            if value_extractor := mapper_config.value_extractor:
                value = value_extractor(value)
            try:
                return mapper_config.mapper(value, type=type)
            except InvalidValueError as e:
                log.info(f"Skipping entity. {e}")
                return
        log.info(f"Skipping entity. No suitable mapper. type=`{type}` value=`{value}`")



import datetime
import logging
import re
from enum import Enum
from typing import List, NamedTuple, Union, Callable

from pytz import UTC
from stix2 import TLP_AMBER, Bundle, ExternalReference, Report

from titan_client.titan_stix.exceptions import TitanStixException
from .entities import EntitiesMapper

from .. import STIXMapperSettings, author_identity, generate_id, StixObjects
from .common import BaseMapper, StixMapper
from ..constants import MARKING, REMOVE_HTML_REGEX
from ..sdo import map_organization


log = logging.getLogger(__name__)


class ReportType(Enum):
    INFOREP = "inforep"
    FINTEL = "fintel"
    BREACH_ALERT = "breach_alert"
    SPOTREP = "spotrep"
    MALWARE = "malware"


class ReportSettings(NamedTuple):
    api_class: str
    method_name: str
    # either JSON path to the field or a function that extracts value from provided source
    title_path: str
    description_path_or_extractor: Union[str, Callable]
    released_at_path: str
    entities_path_or_extractor: Union[str, Callable]
    victims_path: str
    links_path: str
    girs_path: str
    contents_paths: Union[List[str], None] = None



@StixMapper.register("fintels_inforeps", lambda x: "reportTotalCount" in x)
@StixMapper.register("fintel_inforep", lambda x: x.get("documentFamily") in ("FINTEL", "INFOREP"))
@StixMapper.register("breach_alerts", lambda x: "breach_alerts_total_count" in x)
@StixMapper.register("breach_alert", lambda x: "breach_alert" in x.get("data", {}))
@StixMapper.register("spotreps", lambda x: "spotReportsTotalCount" in x)
@StixMapper.register("spotrep", lambda x: "spot_report" in x.get("data", {}))
@StixMapper.register("malreps", lambda x: "malwareReportTotalCount" in x)
@StixMapper.register("malrep", lambda x: "malware_report_data" in x.get("data", {}))
class ReportMapper(BaseMapper):
    reports_settings = {
        ReportType.FINTEL: ReportSettings(
            api_class="ReportsApi",
            method_name="reports_uid_get",
            title_path="subject",
            # There's no summary or anything similar. Try to extract contents of the first
            # paragraph <h2>...</h2><p>get-this</p>...
            description_path_or_extractor=lambda src: re.split(r'</?p>', re.sub(r'^.*?<p>', '', src.get("rawText") or ""))[0],
            released_at_path="created",
            entities_path_or_extractor="entities",
            victims_path="victims",
            links_path="sources",
            contents_paths=["rawText"],
            girs_path = "classification.intelRequirements"
        ),
        ReportType.INFOREP: ReportSettings(
            api_class="ReportsApi",
            method_name="reports_uid_get",
            title_path="subject",
            description_path_or_extractor="executiveSummary",
            released_at_path="created",
            entities_path_or_extractor="entities",
            victims_path="victims",
            links_path="sources",
            contents_paths=["executiveSummary", "researcherComments", "rawText", "rawTextTranslated"],
            girs_path = "classification.intelRequirements"
        ),
        ReportType.BREACH_ALERT: ReportSettings(
            api_class="ReportsApi",
            method_name="breach_alerts_uid_get",
            title_path="data.breach_alert.title",
            description_path_or_extractor="data.breach_alert.summary",
            released_at_path="data.breach_alert.released_at",
            entities_path_or_extractor="data.entities",
            victims_path="data.breach_alert.victim",
            links_path="data.breach_alert.sources",
            girs_path="data.breach_alert.intel_requirements"
        ),
        ReportType.SPOTREP: ReportSettings(
            api_class="ReportsApi",
            method_name="spot_reports_uid_get",
            title_path="data.spot_report.spot_report_data.title",
            description_path_or_extractor="data.spot_report.spot_report_data.text",
            released_at_path="data.spot_report.spot_report_data.released_at",
            entities_path_or_extractor="data.entities",
            victims_path="data.spot_report.spot_report_data.victims",
            links_path="data.spot_report.spot_report_data.links",
            girs_path="data.spot_report.spot_report_data.intel_requirements"
        ),
        ReportType.MALWARE: ReportSettings(
            api_class="ReportsApi",
            method_name="malware_reports_uid_get",
            title_path="data.malware_report_data.title",
            description_path_or_extractor=lambda src: re.split(r'</?p>', re.sub(r'^.*?<p>', '', src.get("data", {}).get("malware_report_data", {}).get("text") or ""))[1],
            released_at_path="data.malware_report_data.released_at",
            entities_path_or_extractor=lambda src: [{"type": "MalwareFamily", "value": src.get("data", {}).get("threat", {}).get("data", {}).get("family")}],
            victims_path="",
            links_path="",
            contents_paths=["data.malware_report_data.text"],
            girs_path = "classification.intelRequirements"
        )
    }

    def __init__(self, settings: STIXMapperSettings):
        super().__init__(settings)
        self.entities_mapper = EntitiesMapper()
        self.cache = {}

    def map(self, source: dict) -> Bundle:
        """
        Main entrypoint for mapping responses from /report, /breachAlerts and /spotReports endpoints
        """
        if "reportTotalCount" in source:
            items = source.get("reports") or []
        elif "breach_alerts_total_count" in source:
            items = source.get("breach_alerts") or []
        elif "spotReportsTotalCount" in source:
            items = source.get("spotReports") or []
        elif "malwareReportTotalCount" in source:
            items = source.get("malwareReports") or []
        else:
            items = [source]

        stix_objects = StixObjects()
        for item in items:
            report_type = self._get_type(item)
            if report_type in (ReportType.FINTEL,
                               ReportType.INFOREP) and self._is_full_report_required():
                # In this case search reports API returns shortened version, without content fields
                # Full version is available only when getting individual report by ID
                stix_objects.extend(self._fetch_and_map_report(report_type, item["uid"]).get())
            else:
                stix_objects.extend(self._map_report(item).get())
        if stix_objects:
            bundle = Bundle(*stix_objects.get(), allow_custom=True)
            return bundle

    def map_report_ioc(self, source: dict, object_refs: StixObjects) -> StixObjects:
        """
        Map report in the format as used when attached to IOCs (in the response of /iocs endpoint).
        This format differs from the one in actual reports endpoint (/reports, /breachAlerts, etc.)
        {"iocs": [{"links": {"reports": [{"uid": "123", "subject": "foo", ... }]}}]}
        """
        titan_id = source["uid"]
        titan_url = source["portalReportUrl"]
        report_type = ReportType[titan_url.split("/")[-2].upper()]
        if self._is_full_report_required():
            return self._fetch_and_map_report(report_type, titan_id, object_refs)

        name = source["subject"]
        time_published = self._format_published(source["released"])
        external_references = [ExternalReference(source_name="Titan URL", url=titan_url)]
        confidence = self.map_confidence(source.get("admiraltyCode"))

        return StixObjects([Report(
            id = self._get_report_id(name, time_published),
            name=name,
            description=name,
            report_types = [report_type.value],
            confidence=confidence,
            published=time_published,
            object_refs=object_refs.get(),
            external_references=external_references,
            created_by_ref=author_identity,
            object_marking_refs=TLP_AMBER,
            custom_properties={"x_intel471_com_uid": titan_id}
        )])

    def _map_report(self, source: dict, object_refs: StixObjects = None) -> StixObjects:
        """
        Map report in the format that is used when getting a report by ID.
        In case of FINTEL and INFOREP (/reports endpoint) there will be extra (possible big) fields.
        Breach alert and Spot report look the same in their long and short representation
        """
        if not object_refs:
            object_refs = StixObjects()
        if entities := self._get_entities(source):
            object_refs.extend(entities.get())
        if victims := self._get_victims(source):
            object_refs.extend(victims.get())

        report_type: ReportType = self._get_type(source)
        if not object_refs:
            log.info("No entities associated with the report. Skipping the report %s/%s",
                     report_type, source["uid"])
            return StixObjects()

        stix_objects = StixObjects([MARKING, author_identity])
        stix_objects.extend(object_refs.get())
        name = self._get_title(source)
        time_published = self._format_published(self._get_released_at(source))
        report_types = [report_type.value]
        labels = self._get_malware_families_names(stix_objects)
        labels.extend(self._get_girs_labels(source))
        if report_type == ReportType.FINTEL:
            report_types.append(source["documentType"].lower())
        report_kwargs = {
            "id": self._get_report_id(name, time_published),
            "name": name,
            "description": self._get_description(source) or name,
            "report_types": report_types,
            "confidence": self.map_confidence(source.get("admiraltyCode") or
                                              source.get("data", {}).get("breach_alert", {})
                                              .get("confidence", {}).get("level")),
            "published": time_published,
            "labels": labels,
            "external_references": self._get_external_references(source),
            "object_refs": object_refs.get(),
            "created_by_ref": author_identity,
            "object_marking_refs": [MARKING],
            "custom_properties": {
                "x_intel471_com_uid": source["uid"],
                "content": self._get_opencti_content(source)
            }
        }
        report = Report(**report_kwargs)
        stix_objects.append(report)
        return stix_objects

    def _fetch_and_map_report(self, report_type: ReportType, report_id: str,
                              object_refs: StixObjects = None) -> StixObjects:
        report_settings = self.reports_settings.get(report_type)
        if report_id not in self.cache:
            api_instance = getattr(self.settings.titan_client,
                                   report_settings.api_class)(self.settings.api_client)
            api_response = getattr(api_instance, report_settings.method_name)(report_id)
            self.cache[report_id] = api_response.to_dict(serialize=True)
        return self._map_report(self.cache[report_id], object_refs)

    def _is_full_report_required(self) -> bool:
        return all([self.settings.titan_client,
                    self.settings.api_client,
                    self.settings.report_full_content])

    @staticmethod
    def _get_report_id(name: str, time_published: str) -> str:
        return generate_id(
            Report,
            name=name.strip().lower(),
            published=time_published
        )

    def _extract_value(self, source: dict, path_or_extractor_name: str):
        report_type = self._get_type(source)
        report_settings = self.reports_settings.get(report_type)
        path_or_extractor = getattr(report_settings, path_or_extractor_name, "")
        if isinstance(path_or_extractor, Callable):
            try:
                return path_or_extractor(source)
            except Exception as e:
                log.warning("Can't extract value from report %s/%s using path/extractor `%s`. %s",
                            report_type, source["uid"], path_or_extractor_name, e)
                return None
        return self._extract_value_by_path(source, path_or_extractor) or None

    @staticmethod
    def _extract_value_by_path(source: dict, path: str):
        for i in path.split("."):
            if not source:
                break
            source = source.get(i, {})
        return source or None

    def _get_external_references(self, source: dict) -> List[ExternalReference]:
        references = [ExternalReference(source_name="Titan URL", url=self._get_url(source))]
        if value := self._extract_value(source, "links_path"):
            for link_source in value:
                if "intel471.com/report" in link_source["url"]:
                    continue
                source_name = f"{link_source.get('source_type', '')} {link_source['type']} - {link_source['title']}".strip()
                external_ref = ExternalReference(url=link_source["url"], source_name=source_name)
                references.append(external_ref)
        return references

    def _get_entities(self, source: dict) -> StixObjects:
        value = self._extract_value(source, "entities_path_or_extractor")
        stix_objects = StixObjects()
        for entities_source in value or []:
            if entity := self.entities_mapper.map(**entities_source):
                stix_objects.append(entity)
        return stix_objects

    def _get_victims(self, source: dict) -> StixObjects:
        stix_objects = StixObjects()
        if value := self._extract_value(source, "victims_path"):
            if isinstance(value, dict):
                value = [value]
            for victim_src in value:
                url = None
                if urls := victim_src.get("urls"):
                    url = urls[0]
                stix_objects.append(
                    map_organization(victim_src["name"], url))
            return stix_objects

    @staticmethod
    def _format_published(epoch_millis: int):
        """
        Formatting datetime object for use as ID contributing property in a same way as it's done
        by OpenCTI to have the same ID here and in OpenCTI.
        """
        parsed = datetime.datetime.fromtimestamp(epoch_millis / 1000, UTC)
        return parsed.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"

    @staticmethod
    def _get_type(source: dict) -> ReportType:
        if source.get("documentFamily") == "FINTEL":
            return ReportType.FINTEL
        if source.get("documentFamily") == "INFOREP":
            return ReportType.INFOREP
        if "breach_alert" in source.get("data", {}):
            return ReportType.BREACH_ALERT
        if "spot_report" in source.get("data", {}):
            return ReportType.SPOTREP
        if "malware_report_data" in source.get("data", {}):
            return ReportType.MALWARE
        raise TitanStixException("Unkown report type")

    def _get_url(self, source: dict) -> str:
        report_type = self._get_type(source)
        report_id = source["uid"]
        return f"https://titan.intel471.com/report/{report_type.value}/{report_id}"

    def _get_released_at(self, source: dict) -> Union[int, None]:
        return self._extract_value(source, "released_at_path") or None

    def _get_girs_labels(self, source: dict) -> List[str]:
        girs_paths = self._extract_value(source, "girs_path") or []
        return self.get_girs_labels(girs_paths)

    def _get_title(self, source: dict) -> Union[str, None]:
        return self._extract_value(source, "title_path") or None

    def _get_description(self, source: dict) -> Union[str, None]:
        value = self._extract_value(source, "description_path_or_extractor")
        if value and isinstance(value, str):
            return re.sub(REMOVE_HTML_REGEX, "", value)

    @staticmethod
    def _get_malware_families_names(entities: StixObjects) -> List[str]:
        return [i.name for i in entities.get() if i.type == "malware"]

    def _get_opencti_content(self, source: dict):
        content_bits = []
        contents_paths = self.reports_settings.get(self._get_type(source)).contents_paths or []
        for path in contents_paths:
            value = self._extract_value_by_path(source, path)
            if value and isinstance(value, str):
                if len(contents_paths) > 1:
                    heading = " ".join([i.capitalize() for i in re.findall(
                        r'[A-Z](?:[a-z]+|[A-Z]*(?=[A-Z]|$))|[a-z]+', path.split(".")[-1])])
                    content_bits.append(f"<h1>{heading}</h1>")
                content_bits.append(re.sub(r"<img src[^>]*>", "", value))
        return "\n".join(content_bits)



import abc
import datetime
import json
import logging
import os
import re
import tempfile
from abc import ABC
from collections import namedtuple
from collections.abc import Callable
from functools import wraps
from typing import Union, List

from stix2 import Bundle

from .. import STIXMapperSettings
from ..exceptions import EmptyBundle, StixMapperNotFound

log = logging.getLogger(__name__)


MappingConfig = namedtuple(
    "MappingConfig",
    ["patterning_mapper", "entities_mapper", "kwargs_extractor", "name_extractor", "opencti_type"]
)


class StixMapper:
    def __init__(self, settings: STIXMapperSettings = None):
        self.settings = settings if settings else STIXMapperSettings()

    mappers = {}

    @classmethod
    def register(cls, name: str, condition) -> Callable:
        """
        Decorator used for registering mapper classes. Decorate any class derived from BaseMapper like this:

        @StixMapper.register("actors", lambda x: "actorTotalCount" in x)
        class ActorsMapper(BaseMapper):
            def map(self, source: dict) -> Bundle:
                ... my implementation ...

        @param name: unique name under which the mapper will be registered
        @param condition: callable that will be called against the source dict
                          to determine if given mapper should be used or not

        """

        def inner_wrapper(wrapped_class: Callable) -> Callable:
            if name in cls.mappers:
                log.info(f"Mapper for {name} already exists. Will replace it")
            cls.mappers[name] = (condition, wrapped_class)
            return wrapped_class

        return inner_wrapper

    def map(self, source: dict, stix_version: str = "2.1") -> Bundle:
        log.info(f"Initializing converter. STIX version {stix_version}.")
        for name, (condition, mapper_class) in self.mappers.items():
            if condition(source):
                log.info(f"Mapping Titan payload for {name}.")
                mapper = mapper_class(self.settings)
                bundle = mapper.map(source)
                if bundle:
                    return bundle
                else:
                    raise EmptyBundle("STIX Mapper produced an empty bundle.")
        raise StixMapperNotFound(
            f"STIX Mapper for this payload is not available (keys: {', '.join(source.keys())})."
        )


def cached(key):
    def decorate(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            ttl_seconds = 10 * 60 * 60  # 10 hours by default
            try:
                ttl_seconds = int(os.getenv("I471_TITAN_CLIENT_CACHE_TTL"))
            except (TypeError, ValueError):
                pass
            if ttl_seconds == 0:
                return func(*args, **kwargs)
            tempdir = tempfile.gettempdir()
            cache_postfix = int(datetime.datetime.utcnow().timestamp() / ttl_seconds)
            cache_path = os.path.join(tempdir, f"{key}{cache_postfix}")
            result = {}
            try:
                with open(cache_path) as fh:
                    result = json.load(fh)
            except Exception as e:
                try:
                    result = func(*args, **kwargs)
                    try:
                        for tmpfile in list(os.walk(tempdir))[0][2]:
                            if tmpfile.startswith(key):
                                os.remove(os.path.join(tempdir, tmpfile))
                    except Exception:
                        pass
                    with open(cache_path, 'w') as fh:
                        json.dump(result, fh)
                except Exception as e:
                    pass
            return result
        return wrapper
    return decorate


class BaseMapper(ABC):
    def __init__(self, settings: STIXMapperSettings):
        self.now = datetime.datetime.utcnow()
        self.settings = settings

    @abc.abstractmethod
    def map(self, source: dict) -> Bundle:
        raise NotImplementedError

    def map_confidence(self, confidence: Union[str, None]):
        # There are two confidence scales used in Titan: https://en.wikipedia.org/wiki/Admiralty_code and low/medium/high
        # This function handles both according to
        # [STIX confidence scales](https://docs.oasis-open.org/cti/stix/v2.1/os/stix-v2.1-os.html#_1v6elyto0uqg)

        # If it's Admiralty_code we're interested in the second part only (Credibility), which is a number between 1 and 6.
        value = re.sub(r"^[A-F]([1-6])$", "\\1", confidence or "", re.IGNORECASE)

        # If there's no match, we expect a word from low/medium/high scale.
        # If for any reason it's not the case either, we set confidence as not specified (`None`)
        return {
            "6": 0, "5": 10, "4": 30, "3": 50, "2": 70, "1": 90,
            "low": 15, "medium": 50, "high": 85
        }.get(value, 0)

    def map_tactic(self, tactic: str):
        if tactic and len(tactic) > 0:
            return tactic.replace("_", "-").replace(" ", "-").lower()

    @staticmethod
    def shorten(text: str, limit: int) -> str:
        if len(text) > limit:
            text = text[:limit]
            while text[-1] != " ":
                text = text[:-1]
        return text.strip()

    @cached("i471titanclientgirs")
    def _get_girs_names(self) -> dict:
        girs_names = {}
        if not all([self.settings.titan_client, self.settings.api_client, self.settings.girs_names]):
            return girs_names
        api_instance = self.settings.titan_client.GIRsApi(self.settings.api_client)
        for offset in range(0, 1000, 100):
            api_response = api_instance.girs_get(count=100, offset=offset)
            if not api_response.girs:
                break
            for gir in api_response.girs:
                girs_names[gir.data.gir.path] = gir.data.gir.name
        return girs_names

    def get_girs_labels(self, gir_paths: List[str]):
        girs_names = self._get_girs_names()
        paths_and_names = {i: girs_names.get(i) for i in gir_paths}
        return [
            f'Intel 471 - GIR {path}{" - " + name if name else ""}'
            for path, name in paths_and_names.items()
        ]



# coding: utf-8

"""
    Titan API v1

    Official low-level client for Intel 471's Titan API. It aims at providing common ground for all the endpoints in Python.

    The version of the OpenAPI document: 1.20.0.8
    Generated by: https://openapi-generator.tech
"""

from pkg_resources import parse_requirements
from setuptools import setup, find_packages  # noqa: H301

NAME = "Titan Client"
VERSION = "1.20.0.8"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

with open("requirements.txt", "r") as requirements_txt:
    reqs = parse_requirements(requirements_txt)
    REQUIRES = [str(req) for req in reqs]

setup(
    name=NAME,
    version=VERSION,
    description="Titan API v1",
    author="Intel 471 Inc.",
    author_email="support@intel471.com",
    license="MIT",
    url="https://github.com/intel471/titan-client-python",
    keywords=["OpenAPI", "OpenAPI-Generator", "Titan API v1"],
    python_requires=">=3.8",
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    long_description_content_type='text/markdown',
    long_description="""\
    Official low-level client for Intel 471's Titan API. It aims at providing common ground for all the endpoints in Python.
    """
)



import json
import re
from unittest.mock import MagicMock, patch

import pytest
import titan_client

from .conftest import PREFIX, read_fixture


configuration = titan_client.Configuration()

test_params = {
    'ActorsApi:actors_get': ('ActorsApi', 'actors_get', {'actor': 'synthx', 'count': 1}, 'SimpleActorsResponse', 'https://api.intel471.com/v1/actors'),
    'ActorsApi:actors_uid_get': ('ActorsApi', 'actors_uid_get', {'uid': '636452233dec0f291302cbafc630c130'}, 'SimpleActorSchema', 'https://api.intel471.com/v1/actors/636452233dec0f291302cbafc630c130'),
    'AlertsApi:alerts_get': ('AlertsApi', 'alerts_get', {'count': 1}, 'AlertListSchemaResponse', 'https://api.intel471.com/v1/alerts'),
    'CredentialsApi:credential_sets_accessed_urls_get': ('CredentialsApi', 'credential_sets_accessed_urls_get', {'text': 'hubspot', 'count': 1}, 'CredentialSetsAccessedUrlsResponse', 'https://api.intel471.com/v1/credentialSets/accessedUrls'),
    'CredentialsApi:credential_sets_accessed_urls_stream_get': ('CredentialsApi', 'credential_sets_accessed_urls_stream_get', {'text': 'hubspot', 'count': 1}, 'CredentialSetsAccessedUrlsStreamResponse', 'https://api.intel471.com/v1/credentialSets/accessedUrls/stream'),
    'CredentialsApi:credential_sets_get': ('CredentialsApi', 'credential_sets_get', {'text': 'leak', 'count': 1}, 'CredentialSetsResponse', 'https://api.intel471.com/v1/credentialSets'),
    'CredentialsApi:credential_sets_stream_get': ('CredentialsApi', 'credential_sets_stream_get', {'text': 'leak', 'count': 1}, 'CredentialSetsStreamResponse', 'https://api.intel471.com/v1/credentialSets/stream'),
    'CredentialsApi:credentials_accessed_urls_get': ('CredentialsApi', 'credentials_accessed_urls_get', {'text': 'hubspot', 'count': 1}, 'CredentialAccessedUrlsResponse', 'https://api.intel471.com/v1/credentials/accessedUrls'),
    'CredentialsApi:credentials_accessed_urls_stream_get': ('CredentialsApi', 'credentials_accessed_urls_stream_get', {'text': 'hubspot', 'count': 1}, 'CredentialAccessedUrlsStreamResponse', 'https://api.intel471.com/v1/credentials/accessedUrls/stream'),
    'CredentialsApi:credentials_get': ('CredentialsApi', 'credentials_get', {'text': 'leak', 'count': 1}, 'CredentialsResponse', 'https://api.intel471.com/v1/credentials'),
    'CredentialsApi:credentials_occurrences_get': ('CredentialsApi', 'credentials_occurrences_get', {'text': 'leak', 'count': 1}, 'CredentialOccurrencesResponse', 'https://api.intel471.com/v1/credentials/occurrences'),
    'CredentialsApi:credentials_occurrences_stream_get': ('CredentialsApi', 'credentials_occurrences_stream_get', {'text': 'leak', 'count': 1}, 'CredentialOccurrencesStreamResponse', 'https://api.intel471.com/v1/credentials/occurrences/stream'),
    'CredentialsApi:credentials_stream_get': ('CredentialsApi', 'credentials_stream_get', {'text': 'leak', 'count': 1}, 'CredentialsStreamResponse', 'https://api.intel471.com/v1/credentials/stream'),
    'EntitiesApi:entities_get': ('EntitiesApi', 'entities_get', {'entity': 'syntax', 'count': 1}, 'EntitiesResponse', 'https://api.intel471.com/v1/entities'),
    'EventsApi:events_get': ('EventsApi', 'events_get', {'count': 1}, 'EventsResponse', 'https://api.intel471.com/v1/events'),
    'EventsApi:events_stream_get': ('EventsApi', 'events_stream_get', {'count': 1}, 'EventStreamResponse', 'https://api.intel471.com/v1/events/stream'),
    'ForumsApi:posts_get': ('ForumsApi', 'posts_get', {'forum': 'opensc.ws', 'count': 1}, 'PostsResponse', 'https://api.intel471.com/v1/posts'),
    'ForumsApi:private_messages_get': ('ForumsApi', 'private_messages_get', {'forum': 'opensc.ws', 'count': 1}, 'PrivateMessagesResponse', 'https://api.intel471.com/v1/privateMessages'),
    'GIRsApi:girs_get': ('GIRsApi', 'girs_get', {'count': 1}, 'GirsResponse', 'https://api.intel471.com/v1/girs'),
    'GlobalSearchApi:search_get': ('GlobalSearchApi', 'search_get', {'text': 'joker', 'count': 1}, 'SearchSchema', 'https://api.intel471.com/v1/search'),
    'IOCsApi:iocs_get': ('IOCsApi', 'iocs_get', {'ioc': '.com', 'count': 1}, 'IocsResponse', 'https://api.intel471.com/v1/iocs'),
    'IndicatorsApi:indicators_get': ('IndicatorsApi', 'indicators_get', {'count': 1}, 'IndicatorSearchResponse', 'https://api.intel471.com/v1/indicators'),
    'IndicatorsApi:indicators_stream_get': ('IndicatorsApi', 'indicators_stream_get', {'count': 1}, 'IndicatorStreamResponse', 'https://api.intel471.com/v1/indicators/stream'),
    'MalwareFamiliesApi:malware_families_get':('MalwareFamiliesApi', 'malware_families_get', {'count': 1}, 'MalwareFamilySearchResponse', 'https://api.intel471.com/v1/malwareFamilies'),
    'MarketplacesApi:marketplaces_get': ('MarketplacesApi', 'marketplaces_get', {'text': 'market', 'count': 1}, 'MarketplaceSearchResponse', 'https://api.intel471.com/v1/marketplaces'),
    'MarketplacesApi:marketplaces_products_get':('MarketplacesApi', 'marketplaces_products_get', {'text': 'visa', 'count': 1}, 'MarketplaceProductSearchResponse', 'https://api.intel471.com/v1/marketplaces/products'),
    'MarketplacesApi:marketplaces_products_stream_get':('MarketplacesApi', 'marketplaces_products_stream_get', {'text': 'visa', 'count': 1}, 'MarketplaceProductStreamResponse', 'https://api.intel471.com/v1/marketplaces/products/stream'),
    'MarketplacesApi:marketplaces_resources_get': ('MarketplacesApi', 'marketplaces_resources_get', {'text': 'visa', 'count': 1}, 'MarketplaceResourceSearchResponse', 'https://api.intel471.com/v1/marketplaces/resources'),
    'MarketplacesApi:marketplaces_resources_stream_get':('MarketplacesApi', 'marketplaces_resources_stream_get', {'text': 'visa', 'count': 1}, 'MarketplaceResourceStreamResponse', 'https://api.intel471.com/v1/marketplaces/resources/stream'),
    'MarketplacesApi:marketplaces_vendors_get': ('MarketplacesApi', 'marketplaces_vendors_get', {'text': 'hydra', 'count': 1}, 'MarketplaceVendorSearchResponse', 'https://api.intel471.com/v1/marketplaces/vendors'),
    'MarketplacesApi:marketplaces_vendors_stream_get': ('MarketplacesApi', 'marketplaces_vendors_stream_get', {'text': 'hydra', 'count': 1}, 'MarketplaceVendorStreamResponse', 'https://api.intel471.com/v1/marketplaces/vendors/stream'),
    'MessagingServicesApi:messaging_services_instant_messages_get': ('MessagingServicesApi', 'messaging_services_instant_messages_get', {'instant_message_service': 'telegram', 'count': 1}, 'MessagingServicesResponse', 'https://api.intel471.com/v1/messagingServices/instantMessages'),
    'NewsApi:news_get': ('NewsApi', 'news_get', {'count': 1}, 'SimpleNewsResponse', 'https://api.intel471.com/v1/news'),
    'NewsApi:news_uid_get': ('NewsApi', 'news_uid_get', {'uid': '1c6c177fb29b4e896b4753a4d7c010b2'}, 'FullNewsSchema', 'https://api.intel471.com/v1/news/1c6c177fb29b4e896b4753a4d7c010b2'),
    'ReportsApi:breach_alerts_get': ('ReportsApi', 'breach_alerts_get', {'breach_alert': 'lock', 'count': 1}, 'SimpleBreachAlertResponse', 'https://api.intel471.com/v1/breachAlerts'),
    'ReportsApi:breach_alerts_uid_get': ('ReportsApi', 'breach_alerts_uid_get', {'uid': 'b43a4f4171a94aee7822dc4bd34b60b3'}, 'SimpleBreachAlertSchema', 'https://api.intel471.com/v1/breachAlerts/b43a4f4171a94aee7822dc4bd34b60b3'),
    'ReportsApi:malware_reports_get': ('ReportsApi', 'malware_reports_get', {'threat_type': 'malware', 'count': 1}, 'MalwareReportsSearchResponse', 'https://api.intel471.com/v1/malwareReports'),
    'ReportsApi:malware_reports_uid_get': ('ReportsApi', 'malware_reports_uid_get', {'uid': '05b23856b7d609aee914b870ea579cd8'}, 'MalwareReportsSearchSchema', 'https://api.intel471.com/v1/malwareReports/05b23856b7d609aee914b870ea579cd8'),
    'ReportsApi:reports_get': ('ReportsApi', 'reports_get', {'count': 1}, 'SimpleReportsResponse', 'https://api.intel471.com/v1/reports'),
    'ReportsApi:reports_uid_get': ('ReportsApi', 'reports_uid_get', {'uid': '36d9c4bec9420d643be1ebff3a6942bf0a6fce9ac1b0cd1376fbf76b00f0ba09'}, 'FullReportSchema', 'https://api.intel471.com/v1/reports/36d9c4bec9420d643be1ebff3a6942bf0a6fce9ac1b0cd1376fbf76b00f0ba09'),
    'ReportsApi:spot_reports_get': ('ReportsApi', 'spot_reports_get', {'spot_report': 'card', 'count': 1}, 'SimpleSpotReportsResponse', 'https://api.intel471.com/v1/spotReports'),
    'ReportsApi:spot_reports_uid_get': ('ReportsApi', 'spot_reports_uid_get', {'uid': '71209453544e87917ffe2e2967f1c385'}, 'FullSpotReportSchema', 'https://api.intel471.com/v1/spotReports/71209453544e87917ffe2e2967f1c385'),
    'ReportsApi:situation_reports_get': ('ReportsApi', 'situation_reports_get', {'situation_report': 'malware', 'count': 1}, 'SituationReportResponse', 'https://api.intel471.com/v1/situationReports'),
    'ReportsApi:situation_reports_report_uid_get': ('ReportsApi', 'situation_reports_report_uid_get', {'report_uid': '517fc6de76929e70eeb3843b63d5ce41'}, 'SituationReportSchema', 'https://api.intel471.com/v1/situationReports/517fc6de76929e70eeb3843b63d5ce41'),
    'TagsApi:tags_get': ('TagsApi', 'tags_get', {}, 'TagResponse', 'https://api.intel471.com/v1/tags'),
    'VulnerabilitiesApi:cve_reports_get': ('VulnerabilitiesApi', 'cve_reports_get', {'count': 1}, 'SimpleCvesResponse', 'https://api.intel471.com/v1/cve/reports'),
    'VulnerabilitiesApi:cve_reports_uid_get': ('VulnerabilitiesApi', 'cve_reports_uid_get', {'uid': '6b95e65d42a546629cb0a23bc022ecde'}, 'FullCveSchema', 'https://api.intel471.com/v1/cve/reports/6b95e65d42a546629cb0a23bc022ecde'),
    'WatchersApi:watcher_groups_get': ('WatchersApi', 'watcher_groups_get', {}, 'WatcherGroupResponse', 'https://api.intel471.com/v1/watcherGroups'),
    'WatchersApi:watcher_groups_group_uid_get': ('WatchersApi', 'watcher_groups_group_uid_get', {'group_uid': '3d0b4432-7390-4ae5-92be-52c3e39dc775'}, 'SimpleWatcherGroupSchema', 'https://api.intel471.com/v1/watcherGroups/3d0b4432-7390-4ae5-92be-52c3e39dc775'),
    'WatchersApi:watcher_groups_group_uid_watchers_get': ('WatchersApi', 'watcher_groups_group_uid_watchers_get', {'group_uid': '3d0b4432-7390-4ae5-92be-52c3e39dc775'}, 'WatcherSchemaResponse', 'https://api.intel471.com/v1/watcherGroups/3d0b4432-7390-4ae5-92be-52c3e39dc775/watchers'),
    'WatchersApi:watcher_groups_group_uid_watchers_watcher_uid_get': ('WatchersApi', 'watcher_groups_group_uid_watchers_watcher_uid_get', {'group_uid': '3d0b4432-7390-4ae5-92be-52c3e39dc775', 'watcher_uid': '81808755feef2000acb593a3a5239cdc'}, 'WatcherSchema', 'https://api.intel471.com/v1/watcherGroups/3d0b4432-7390-4ae5-92be-52c3e39dc775/watchers/81808755feef2000acb593a3a5239cdc'),
    'YARAApi:yara_get': ('YARAApi', 'yara_get', {'threat_type': 'malware', 'count': 1}, 'YARASearchResponse', 'https://api.intel471.com/v1/yara'),
}

@patch('titan_client.rest.RESTClientObject')
@pytest.mark.parametrize('api_cls_name, method_name, kwargs, filename, query_url', test_params.values(), ids=test_params.keys())
def test_api_responses(rest_client_class_mock, api_cls_name, method_name, kwargs, filename, query_url):
    rest_client_response = MagicMock()
    rest_client_response.status = 200
    rest_client_response.reason = 'OK'
    rest_client_response.getheader.return_value = 'application/json; charset=utf-8'
    response = read_fixture(f'{PREFIX}/fixtures/api_responses/{filename}.json')
    rest_client_response.data = json.dumps(response).encode('utf-8')

    rest_client_instance_mock = MagicMock()
    rest_client_instance_mock.GET.return_value = rest_client_response

    rest_client_class_mock.side_effect = [rest_client_instance_mock]

    with titan_client.ApiClient(configuration) as api_client:
        api_instance = getattr(titan_client, api_cls_name)(api_client)
        api_response = getattr(api_instance, method_name)(**kwargs)

        if 'uid' in method_name:
            expected_params = []
        else:
            kwargs_camel_case = [(re.sub('_.',lambda x: x.group()[1].upper(), k), v) for k,v in kwargs.items()]
            expected_params = kwargs_camel_case

        assert rest_client_instance_mock.GET.call_args_list[0][0][0] == query_url
        assert rest_client_instance_mock.GET.call_args_list[0][1]['query_params'] == expected_params
        assert api_response.to_dict(serialize=True) == response



def _wrap(*components):
    return "[" + " OR ".join([f"{i} = {j}" for i, j in components]) + "]"


def create_url_pattern(value: str) -> str:
    return _wrap(("url:value", f"'{value}'"))


def create_ipv4_pattern(value: str) -> str:
    return _wrap(("ipv4-addr:value", f"'{value}'"))


def create_domain_pattern(value: str) -> str:
    return _wrap(("domain-name:value", f"'{value}'"))


def create_file_pattern(md5: str = None, sha1: str = None, sha256: str = None) -> str:
    """
    Using format `file:hashes.md5`, `file:hashes.sha1`, etc.
    instead of   `file:hashes.MD5`, `file:hashes.'SHA-1'`, etc.
    to be compliant with OpenCTI ID generator, which is using the former one.
    """
    hashes = []
    if md5:
        hashes.append(("file:hashes.md5", f"'{md5}'"))
    if sha1:
        hashes.append(("file:hashes.sha1", f"'{sha1}'"))
    if sha256:
        hashes.append(("file:hashes.sha256", f"'{sha256}'"))
    return _wrap(*hashes)
